import express from "express";

import db from "../../lib/sql";

const queries = {
  getlangs: "select * from `languages` order by `name` asc",
  add:      "insert into `pastes` (`uuid`,`lang`,`desc`,`paste`,`hidden`,`reply`) "
          + "values (LEFT(UUID(), 8), ?, ?, ?, ?, ?)",
  getuuid:  "select `uuid` from `pastes` where `id` = ? limit 1",
  getid:    "select `id` from `pastes` where `uuid` = ? limit 1"
};
let languages = new Map();
db.exec(queries.getlangs)
  .then(rows => rows.forEach(r => languages.set(r.id, r.name)))
  .catch(err => console.log(err));

export default express.Router()
  .get(["/", "/r/:uuid"], (req, res) => res.render("index", { languages: [...languages.values()], reply: req.params.uuid || false }))
  .post("/", (req, res) => {
    let data = {
      desc: req.body.desc || "",
      content: req.body.content || false,
      lang: languages.getKey(req.body.lang) || 1,
      hidden: req.body.hidden === "on" || false,
      reply: req.body.reply || 0
    };
    if(!data.content)
      return res.render("index", { languages: [...languages.values()], notification: "nope" });

    if(data.reply.length === 8) {
      db.exec(queries.getid, [data.reply]).then(rows => {
        data.reply = rows.length === 1 ? rows[0].id : 0;
        savePaste(data, res);
      }).catch(err => console.log(err));
    }
    else {
      data.reply = 0;
      savePaste(data, res);
    }
  });

const savePaste = (data, res) => {
  db.exec(queries.add, [
    data.lang,
    data.desc,
    new Buffer(data.content).toString("base64"),
    data.hidden,
    data.reply
  ]).then(insert => db.exec(queries.getuuid, [insert.insertId])
    .then(rows => res.redirect(`/${rows[0].uuid}/`))
    .catch(err => res.render("index", { languages: [...languages.values()], notification: "lol" }))
  ).catch(err => res.render("index", { languages: [...languages.values()], notification: "didn't insert your paste lol" }));
};

Map.prototype.getKey = function(val) {
  if(!val)
    return 1;
  for(let [key, value] of this)
    if(value.toLowerCase() === val.toLowerCase())
      return key;
  return false;
};