import express from "express";

import highlight from "highlight.js";
import db from "../../lib/sql";

const queries = {
  all: filter => "select p.id, p.uuid, p.stamp, p.desc, p.paste, p.hidden, l.name as language, r.uuid as reply from `pastes` as p "
          + "left join `languages` as l on l.id = p.lang "
          + "left join `pastes` as r on r.id = p.reply "
          + "where p.hidden = 0 "
          + (filter || "")
          + "order by p.id desc limit 5",
  single: "select p.id, p.uuid, p.stamp, p.desc, p.paste, l.name as language, r.uuid as reply from `pastes` as p "
          + "left join `languages` as l on l.id = p.lang "
          + "left join `pastes` as r on r.id = p.reply "
          + "where p.uuid = ? limit 1"
};

export default express.Router()
  .get("/", (req, res) => {
    db.exec(queries.all())
      .then(rows => {
        res.render("view", { pastes: rows.map(r => {
          const paste = new Buffer(r.paste, "base64").toString("ascii");
          return {
            uuid: r.uuid,
            paste: highlight.highlightAuto(paste).value,
            language: r.language,
            stamp: r.stamp,
            desc: r.reply ? `Reply to <a href="/${r.reply}/" class="descID">/${r.reply}/</a>` : r.desc || "No description.",
            rows: paste.split(/\r?\n/).length
          };
        })});
      }).catch(err => res.render("view"));
  })
  .get("/:uuid", (req, res) => {
    if(req.params.uuid.length !== 8)
      return res.send("error");
    db.exec(queries.single, [req.params.uuid])
      .then(rows => {
        const r = rows[0];
        const paste = new Buffer(r.paste, "base64").toString("ascii");
        res.render("viewsingle", { paste: {
          uuid: r.uuid,
          paste: highlight.highlightAuto(paste).value,
          language: r.language,
          stamp: r.stamp,
          desc: r.reply ? `Reply to <a href="/${r.reply}/" class="descID">/${r.reply}/</a>` : r.desc || "No description.",
          rows: paste.split(/\r?\n/).length
        }});
      }).catch(err => res.send("paste not found"));
  })
  .get("/:filter(lang|desc)/:fvalue", (req, res) => {
    const condition = {
      desc: "and p.desc like ? ",
      lang: "and l.name = ? "
    };
    let value = req.params.fvalue;
    if(req.params.filter == "desc")
      value = "%" + value + "%";
    db.exec(queries.all(condition[req.params.filter]), value)
      .then(rows => {
        res.render("view", { pastes: rows.map(r => {
          const paste = new Buffer(r.paste, "base64").toString("ascii");
          return {
            uuid: r.uuid,
            paste: highlight.highlightAuto(paste).value,
            language: r.language,
            stamp: r.stamp,
            desc: r.reply ? `Reply to <a href="/${r.reply}/" class="descID">/${r.reply}/</a>` : r.desc || "No desciption.",
            rows: paste.split(/\r?\n/).length
          };
        })});
      }).catch(err => res.render("view"));
  })
  .post("/", (req, res) => {
    const desc = /^"(.*)"$/;
    let match;
    if(match = desc.exec(req.body.filter))
      res.redirect("/v/desc/" + encodeURIComponent(match[1]));
    else
      res.redirect("/v/lang/" + encodeURIComponent(req.body.filter));
  });
